--[[-----------------------------------------------------------------------------
Healcsi ScrollFrame Container
Plain container that scrolls its content and doesn't grow in height.
-------------------------------------------------------------------------------]]
local Healcsi = Healcsi
local TopParent = Healcsi.UIParent

local Type, Version = "Healcsi-ScrollFrame", 1
local AceGUI = LibStub and LibStub("AceGUI-3.0", true)
if not AceGUI or (AceGUI:GetWidgetVersion(Type) or 0) >= Version then return end

-- Lua APIs
local pairs, assert, type = pairs, assert, type
local min, max, floor, abs = math.min, math.max, math.floor, math.abs

-- WoW APIs
local CreateFrame = CreateFrame

--[[-----------------------------------------------------------------------------
Support functions
-------------------------------------------------------------------------------]]
local function FixScrollOnUpdate(frame)
	frame:SetScript("OnUpdate", nil)
	frame.obj:FixScroll()
end

--[[-----------------------------------------------------------------------------
Scripts
-------------------------------------------------------------------------------]]
local function ScrollFrame_OnMouseWheel(frame, value)
	frame.obj:MoveScroll(value)
end

local function ScrollFrame_OnSizeChanged(frame)
	frame:SetScript("OnUpdate", FixScrollOnUpdate)
end

local function ScrollBar_OnScrollValueChanged(frame, value)
	frame.obj:SetScroll(value)
end

local function ScrollBar_OnMouseDown(frame)
	frame.animationShrinkLock = true
end

local function ScrollBar_OnMouseUp(frame)
	frame.animationShrinkLock = nil
	if frame.animationShouldShrink then
		frame.animationShouldShrink = nil
		frame.animation = "shrink"
	end
end

local function ScrollBar_OnEnter(frame)
	frame.animation = "grow"
end

local function ScrollBar_OnLeave(frame)
	if (frame.animationShrinkLock) then
		frame.animationShouldShrink = true
	else
		frame.animation = "shrink"
	end

end

local function Frame_Update(frame, elapsed)
	if (frame.scrollbar.animation) then
		frame.elapsed = (frame.elapsed or 0) + elapsed
		if (frame.elapsed > (frame.animSpeed or 0.01)) then
			frame.elapsed = 0

			local localDb = frame.obj.localDb

			--Healcsi:Print("ANIM: " .. frame.scrollbar.animation)

			if (frame.scrollbar.animation == "grow") then

				if (frame.growDelay and frame.growDelay > 0) then
					frame.growDelay = frame.growDelay - 1
					return
				end

				frame.shrinkDelay = 2
				
				local targetWidth = localDb.width + localDb.grow

				for i=1,10 do

					local currentWidth = frame.scrollbar:GetWidth()

					if (targetWidth > currentWidth) then
						currentWidth = currentWidth + 0.1
						frame.scrollbar:SetWidth(currentWidth)
						frame.scrollbar.thumb:SetWidth(currentWidth)
					else
						frame.scrollbar.animation = nil
						return
					end

				end


			elseif (frame.scrollbar.animation == "shrink") then

				if (frame.shrinkDelay and frame.shrinkDelay > 0) then
					frame.shrinkDelay = frame.shrinkDelay - 1
					return
				end

				frame.growDelay = 2

				local targetWidth = localDb.width

				for i=1,10 do

					local currentWidth = frame.scrollbar:GetWidth()

					if (targetWidth < currentWidth) then
						currentWidth = currentWidth - 0.1
						frame.scrollbar:SetWidth(currentWidth)
						frame.scrollbar.thumb:SetWidth(currentWidth)						
					else
						frame.scrollbar.animation = nil
					end

				end

			end
			
		end
	end
end

--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local methods = {
	["OnAcquire"] = function(self)

		self.localDb = {
			scrollvalue = 0,
			width = 6,
			grow = 2,
			growSide = "right",
			thumbHeight = 75
		}

		self:SetScroll(0)
		self.scrollframe:SetScript("OnUpdate", FixScrollOnUpdate)

		self:SetScrollBarWidth(8)
		self:SetScrollBarBackroundColor(Healcsi:GetColor("DarkBlue", 0.2))
		self:SetScrollBarThumbColor(Healcsi:GetColor("DarkBlue", 1))
	end,

	["OnRelease"] = function(self)
		self.localDb = nil

		self.scrollframe:SetPoint("BOTTOMRIGHT")
		self.scrollbar:Hide()
		self.scrollBarShown = nil
		self.content.height, self.content.width = nil, nil
	end,

	["SetDatabase"] = function(self, db)
		if (db) then
			self.db = db
		else
			self.db = Healcsi.db.profile.Editor.scroll
		end
		if self.db.width then
			self.localDb.width = self.db.width
			self.scrollbar:SetWidth(self.db.width)
			self.scrollbar.thumb:SetWidth(self.db.width)
		end
		if self.db.grow then
			self.localDb.grow = self.db.grow
			self:SetGrowSide(self.localDb.growSide)
		end
		if self.db.thumbHeight then
			self.localDb.thumbHeight = self.db.thumbHeight
			self.scrollbar.thumb:SetHeight(self.db.thumbHeight)
		end

		self:FixScroll()
	end,

	["SetScrollBarMargin"] = function(self, top, right, bottom, left)
		self.localDb.margin = self.localDb.margin or {}

		self.localDb.margin.top = top
		self.localDb.margin.right = right
		self.localDb.margin.bottom = bottom
		self.localDb.margin.left = left

		self:SetGrowSide(self.localDb.growSide)
		--self:FixScroll()
	end,

	["SetGrowSide"] = function(self, side)
		if (side == "right" or side == "left" or side == "center") then

			local db = self.localDb
			local margin = db.margin or {top = 0, right = 0, bottom = 0, left = 0}
			self.scrollbar:ClearAllPoints()

			if (side == "right") then
				self.scrollbar:SetPoint("TOPLEFT", self.scrollframe, "TOPRIGHT", margin.left, -margin.top)
				self.scrollbar:SetPoint("BOTTOMLEFT", self.scrollframe, "BOTTOMRIGHT", margin.left, margin.bottom)
			elseif (side == "left") then
				self.scrollbar:SetPoint("TOPRIGHT", self.frame, "TOPRIGHT", -margin.right, -margin.top)
				self.scrollbar:SetPoint("BOTTOMRIGHT", self.frame, "BOTTOMRIGHT", -margin.right, margin.bottom)
			elseif (side == "center") then
				local c = (db.width + db.grow + margin.left + margin.right)/2
				self.scrollbar:SetPoint("TOP", self.scrollframe, "TOPRIGHT", c, -margin.top)
				self.scrollbar:SetPoint("BOTTOM", self.scrollframe, "BOTTOMRIGHT", c, margin.bottom)
			end
			self.localDb.growSide = side
			self:FixScroll()
		end
	end,

	["GetScrollBarWidth"] = function(self)
		local db = self.localDb
		local margin = db.margin or {top = 0, right = 0, bottom = 0, left = 0}
		return (db.width + db.grow + margin.left + margin.right)
	end,

	["SetScrollBarWidth"] = function(self, width)
		self.scrollbar:SetWidth(width)
		self.scrollbar.thumb:SetWidth(width)
		if (self.localDb) then
			self.localDb.width = width
		end
		if (self.db) then self.db.width = width end

		self:FixScroll()
	end,

	["SetGrowSize"] = function(self, grow)
		self.localDb.grow = grow
		if (self.db) then self.db.grow = grow end

		self:FixScroll()
	end,

	["SetScrollBarBackroundTexture"] = function(self, texture)
		self.scrollbar.background:SetTexture(Healcsi:GetMedia("statusbar", texture))
	end,

	["SetScrollBarBackroundColor"] = function(self, r, g, b, a)
		self.scrollbar.background:SetVertexColor(r, g, b, a)
	end,

	["SetScrollBarThumbTexture"] = function(self, texture)
		self.scrollbar.thumb:SetTexture(Healcsi:GetMedia("statusbar", texture))
	end,

	["SetScrollBarThumbColor"] = function(self, r, g, b, a)
		self.scrollbar.thumb:SetVertexColor(r, g, b, a or 1)
	end,

	["ScrollBarThumbHeight"] = function(self, height)
		self.scrollbar.thumb:SetHeight(height)
		self.localDb.thumbHeight = height
		if (self.db) then self.db.thumbHeight = height end
	end,

	["SetScroll"] = function(self, value)
		local localDb = self.localDb
		local viewheight = self.scrollframe:GetHeight()
		local height = self.content:GetHeight()
		local offset

		if viewheight > height then
			offset = 0
		else
			offset = floor((height - viewheight) / 1000.0 * value)
		end
		self.content:ClearAllPoints()
		self.content:SetPoint("TOPLEFT", 0, offset)
		self.content:SetPoint("TOPRIGHT", 0, offset)

		if (localDb) then
			localDb.offset = offset
			localDb.scrollvalue = value
		end
	end,

	["MoveScroll"] = function(self, value)
		local localDb = self.localDb
		local height, viewheight = self.scrollframe:GetHeight(), self.content:GetHeight()
		
		if self.scrollBarShown then
			local diff = height - viewheight
			local delta = 1
			if value < 0 then
				delta = -1
			end
			self.scrollbar:SetValue(min(max(localDb.scrollvalue + delta*(1000/(diff/45)),0), 1000))
		end
	end,

	["FixScroll"] = function(self)
		if self.updateLock then return end
		self.updateLock = true
		local localDb = self.localDb
		local height, viewheight = self.scrollframe:GetHeight(), self.content:GetHeight()
		local offset = 0
		if (self.localDb) then offset = self.localDb.offset or 0 end
		local curvalue = self.scrollbar:GetValue()
		-- Give us a margin of error of 2 pixels to stop some conditions that i would blame on floating point inaccuracys
		-- No-one is going to miss 2 pixels at the bottom of the frame, anyhow!
		if viewheight < height + 2 then
			if self.scrollBarShown then
				self.scrollBarShown = nil
				self:Fire("ScrollBarOnHidden")
				self.scrollbar:Hide()
				self.scrollbar:SetValue(0)
				self.scrollframe:SetPoint("BOTTOMRIGHT")
				self:DoLayout()
			end
		else
			if not self.scrollBarShown then
				self.scrollBarShown = true
				self:Fire("ScrollBarOnShow")
				self.scrollbar:Show()
				if (self.localDb) then
					local margin = self.localDb.margin or {top = 0, right = 0, bottom = 0, left = 0}
					self.scrollframe:SetPoint("BOTTOMRIGHT", -1*(self.localDb.width + self.localDb.grow + margin.left + margin.right), 0)
				else
					local margin = {top = 0, right = 0, bottom = 0, left = 0}
					self.scrollframe:SetPoint("BOTTOMRIGHT", -1*(6 + 2 + margin.left + margin.right), 0)
				end
				self:DoLayout()
			end
			local value = (offset / (viewheight - height) * 1000)
			if value > 1000 then value = 1000 end
			self.scrollbar:SetValue(value)
			self:SetScroll(value)
			if value < 1000 then
				self.content:ClearAllPoints()
				self.content:SetPoint("TOPLEFT", 0, offset)
				self.content:SetPoint("TOPRIGHT", 0, offset)
				if (self.localDb) then
					self.localDb.offset = offset
				end
			end
		end
		self.updateLock = nil
	end,

	["LayoutFinished"] = function(self, width, height)
		self.content:SetHeight(height or 0 + 20)
		self.scrollframe:SetScript("OnUpdate", FixScrollOnUpdate)
	end,

	["OnWidthSet"] = function(self, width)
		local content = self.content
		content.width = width
	end,

	["OnHeightSet"] = function(self, height)
		local content = self.content
		content.height = height
	end
}
--[[-----------------------------------------------------------------------------
Constructor
-------------------------------------------------------------------------------]]
local function Constructor()
	local frame = CreateFrame("Frame", nil, TopParent)
	local num = AceGUI:GetNextWidgetNum(Type)

	local localDb = {
		scrollvalue = 0,
		width = 6,
		grow = 2,
		growSide = "right",
		thumbHeight = 75
	}

	local scrollframe = CreateFrame("ScrollFrame", nil, frame)
	frame.scrollframe = scrollframe
	scrollframe:SetPoint("TOPLEFT")
	scrollframe:SetPoint("BOTTOMRIGHT")
	scrollframe:EnableMouseWheel(true)
	scrollframe:SetScript("OnMouseWheel", ScrollFrame_OnMouseWheel)
	scrollframe:SetScript("OnSizeChanged", ScrollFrame_OnSizeChanged)

	local scrollbar = CreateFrame("Slider", ("HealcsiScrollFrame%dScrollBar"):format(num), scrollframe)
	frame.scrollbar = scrollbar

	local margin = {}
	margin.left = 0
	margin.right = 0
	margin.top = 0
	margin.bottom = 0

	scrollbar:SetPoint("TOPLEFT", scrollframe, "TOPRIGHT", left, top)
	scrollbar:SetPoint("BOTTOMLEFT", scrollframe, "BOTTOMRIGHT", left, bottom)

	scrollbar:SetMinMaxValues(0, 1000)
	scrollbar:SetValueStep(1)
	scrollbar:SetValue(0)
	scrollbar:SetWidth(8)
	scrollbar:Hide()
	-- set the script as the last step, so it doesn't fire yet
	scrollbar:SetScript("OnValueChanged", ScrollBar_OnScrollValueChanged)

	scrollbar.animation = nil
	scrollbar:SetScript('OnMouseDown', ScrollBar_OnMouseDown)
	scrollbar:SetScript('OnMouseUp', ScrollBar_OnMouseUp)
	scrollbar:SetScript('OnEnter', ScrollBar_OnEnter)
	scrollbar:SetScript('OnLeave', ScrollBar_OnLeave)
	frame:SetScript('OnUpdate', Frame_Update)

	local thumb = scrollbar:CreateTexture(nil, "OVERLAY")
	scrollbar.thumb = thumb
	thumb:SetTexture(Healcsi:GetMedia("statusbar", "Solid"))
	thumb:SetSize(8, 75)
	scrollbar:SetThumbTexture(thumb)

	local background = scrollbar:CreateTexture(nil, "BACKGROUND")
	frame.scrollbar.background = background
	background:SetAllPoints(scrollbar)
	background:SetTexture(Healcsi:GetMedia("statusbar", "Solid"))
	background:SetVertexColor(0, 0, 0, 0)

	--Container Support
	local content = CreateFrame("Frame", nil, scrollframe)
	content:SetPoint("TOPLEFT")
	content:SetPoint("TOPRIGHT")
	content:SetHeight(400)
	scrollframe:SetScrollChild(content)

	local widget = {
		localDb     = localDb,
		scrollframe = scrollframe,
		scrollbar   = scrollbar,
		content     = content,
		frame       = frame,
		type        = Type
	}
	for method, func in pairs(methods) do
		widget[method] = func
	end
	scrollframe.obj, scrollbar.obj = widget, widget

	return AceGUI:RegisterAsContainer(widget)
end

AceGUI:RegisterWidgetType(Type, Constructor, Version)
