--[[-----------------------------------------------------------------------------
Label Widget
Displays text and optionally an icon.
-------------------------------------------------------------------------------]]
local Healcsi      = Healcsi
local Gui          = Healcsi.Gui
local TopParent    = Healcsi.UIParent
local HiddenParent = Healcsi.HiddenParent

local Type, Version = "Healcsi-DebuffItem", 1
local AceGUI = LibStub and LibStub("AceGUI-3.0", true)
if not AceGUI or (AceGUI:GetWidgetVersion(Type) or 0) >= Version then return end

-- Lua APIs
local max, select, pairs = math.max, select, pairs

-- WoW APIs
local CreateFrame = CreateFrame

-- Global vars/functions that we don't upvalue since they might get hooked, or upgraded
-- List them here for Mikk's FindGlobals script
-- GLOBALS: GameFontHighlightSmall

--[[-----------------------------------------------------------------------------
Support functions
-------------------------------------------------------------------------------]]

local function fixlevels(parent,...)
	local i = 1
	local child = select(i, ...)
	while child do
		child:SetFrameLevel(parent:GetFrameLevel()+1)
		fixlevels(child, child:GetChildren())
		i = i + 1
		child = select(i, ...)
	end
end

local function fixstrata(strata, parent, ...)
	local i = 1
	local child = select(i, ...)
	parent:SetFrameStrata(strata)
	while child do
		fixstrata(strata, child, child:GetChildren())
		i = i + 1
		child = select(i, ...)
	end
end


local function UpdateImageAnchor(self)
	if self.resizing then return end
	local frame = self.frame
	local width = frame.width or frame:GetWidth() or 0
	local image = self.image
	local label = self.label
	local height

	label:ClearAllPoints()
	image:ClearAllPoints()

	if self.imageshown then
		local imagewidth = image:GetWidth()
		if (width - imagewidth) < 200 or (label:GetText() or "") == "" then
			-- image goes on top centered when less than 200 width for the text, or if there is no text
			image:SetPoint("TOP")
			label:SetPoint("TOP", image, "BOTTOM")
			label:SetPoint("LEFT")
			label:SetWidth(width)
			height = image:GetHeight() + label:GetHeight()
		else
			-- image on the left
			image:SetPoint("TOPLEFT")
			if image:GetHeight() > label:GetHeight() then
				label:SetPoint("LEFT", image, "RIGHT", 4, 0)
			else
				label:SetPoint("TOPLEFT", image, "TOPRIGHT", 4, 0)
			end
			label:SetWidth(width - imagewidth - 4)
			height = max(image:GetHeight(), label:GetHeight())
		end
	else
		-- no image shown
		label:SetPoint("TOPLEFT")
		label:SetWidth(width)
		height = label:GetHeight()
	end
	
	self.resizing = true
	frame:SetHeight(height)
	frame.height = height
	self.resizing = nil
end

--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local methods = {
	["OnAcquire"] = function(self)
		-- set the flag to stop constant size updates
		self.resizing = true
		-- height is set dynamically by the text and image size
		self:SetWidth(200)

		local prioList = {
			[5] = "Critical",
			[4] = "Very high",
			[3] = "High",
			[2] = "Normal",
			[1] = "Low",
			[0] = "Ignore"
		}

		local dropDown = Gui:Create("Healcsi-Dropdown")
		self.dropDown = dropDown
		dropDown.obj = self
		dropDown:SetWidth(150)
		dropDown:SetList(prioList)
		dropDown:ClearAllPoints()
		dropDown:SetPoint("RIGHT", self.frame, "RIGHT")

		dropDown.dropdown:SetParent(self.frame) -- move front so ponies can survive!!

		self:SetFont(Healcsi:GetFont("Friz Quadrata TT")) -- "MoonBold"
		self:SetFontColor(Healcsi:GetColor("Black", 1))

		self:SetShadowColor(Healcsi:GetColor("DarkBlue", 0.5)) -- BlueSelector
		self:SetShadowOffset(1, -1)

		self:SetText()
		self:SetImage(nil)
		self:SetImageSize(20, 20)
		self:SetJustifyH("LEFT")
		self:SetJustifyV("MIDDLE")

		Healcsi:RegisterEventCallback("HealcsiGui-DebuffItem-ClearFocus", function() -- there can be only one of this so i don't care, hide all!
			self.frame.highlight.state = "normal"

			self.frame.highlight.texture:SetVertexColor(1, 1, 1, 0.3)
			self.frame.highlight.texture:Hide()
		end)

		-- reset the flag
		self.resizing = nil
		-- run the update explicitly
		UpdateImageAnchor(self)
	end,

	["OnRelease"] = function(self)
		Gui:Release(self.dropDown)

		--local dropDown = self.dropDown
		--if dropDown.OnRelease then dropDown:OnRelease() end

		--dropDown.frame:ClearAllPoints()
		--dropDown.frame:Hide()
		--dropDown.frame:SetParent(TopParent)

		Healcsi:BroadcastEvent("HealcsiGui-DebuffItem-ClearFocus")
	end,

	["OnWidthSet"] = function(self, width)
		UpdateImageAnchor(self)
	end,

	["SetText"] = function(self, text)
		self.label:SetText(text)
		UpdateImageAnchor(self)
	end,

	["SetFont"] = function(self, font, height, flags)
		self.label:SetFont(Healcsi:GetMedia("font", font) or font, height or 14, flags)
	end,

	["SetFontColor"] = function(self, r, g, b, a)
		self.label:SetVertexColor(r or 0, g or 0, b or 0, a or 1)
	end,

	["SetShadowColor"] = function(self, r, g, b, a)
		self.label:SetShadowColor(r or 0, g or 0, b or 0, a or 1)
	end,

	["SetShadowOffset"] = function(self, x, y)
		self.label:SetShadowOffset(x or 0, y or 0)
	end,

	["SetImage"] = function(self, path, ...)
		local image = self.image
		image:SetTexture(path)
		
		if image:GetTexture() then
			self.imageshown = true
			local n = select("#", ...)
			if n == 4 or n == 8 then
				image:SetTexCoord(...)
			else
				image:SetTexCoord(0, 1, 0, 1)
			end
		else
			self.imageshown = nil
		end
		UpdateImageAnchor(self)
	end,

	["SetPriority"] = function(self, priority)
		self.dropDown:SetValue(priority)
		if (priority > 4) then
			self.dropDown:SetSelectorFontColor(Healcsi:GetColor("Red", 0.9))
		elseif (priority > 3) then
			self.dropDown:SetSelectorFontColor(Healcsi:GetColor("Orange", 0.9))
		elseif (priority == 0) then
			self.dropDown:SetSelectorFontColor(Healcsi:GetColor("Gray", 0.9))
		else
			self.dropDown:SetSelectorFontColor(Healcsi:GetColor("White", 0.9))
		end
	end,

	["SetImageSize"] = function(self, width, height)
		self.image:SetWidth(width)
		self.image:SetHeight(height)
		local zoom = 0.08
		self.image:SetTexCoord(zoom, 1-zoom, zoom, 1-zoom)
		UpdateImageAnchor(self)
	end,

	["SetJustifyH"] = function(self, justifyH)
		self.label:SetJustifyH(justifyH)
	end,

	["SetJustifyV"] = function(self, justifyV)
		self.label:SetJustifyV(justifyV)
	end
}

--[[-----------------------------------------------------------------------------
Constructor
-------------------------------------------------------------------------------]]
local function Constructor()
	local name = "HealcsiGui-DebuffItem" .. Gui:GetNextWidgetNum(Type)
	local frame = CreateFrame("Frame", name, TopParent)
	frame:Hide()

	local label = frame:CreateFontString(nil, "ARTWORK")
	local image = frame:CreateTexture(nil, "ARTWORK")

	local highlight = CreateFrame("Frame", name.."Highlight", frame)
	frame.highlight = highlight
	highlight.obj = frame
	highlight:SetAllPoints(frame)
	highlight.state = "normal"

	local hTex = frame:CreateTexture(nil, "BACKGROUND")
	frame.highlight.texture = hTex
	hTex:SetPoint("TOPLEFT", -5, 5)
	hTex:SetPoint("BOTTOMRIGHT", -150, -5)
	hTex:SetTexture("Interface\\Buttons\\UI-Listbox-Highlight")
	hTex:SetVertexColor(1, 1, 1, 0.3)
	hTex:Hide()

	highlight:EnableMouse(true)
	highlight:SetScript("OnEnter", function(self)
		self.texture:Show()
		if (self.state ~= "selected") then
			self.texture:SetVertexColor(1, 1, 1, 0.3)
		end
	end)

	highlight:SetScript("OnLeave", function(self)
		if (self.state ~= "selected") then
			self.texture:Hide()
		end
	end)

	-- create widget
	local widget = {
		label = label,
		image = image,
		frame = frame,
		type  = Type
	}

	highlight.fireObj = widget -- it's wierd but working... so... shut up!

	highlight:SetScript("OnMouseUp", function(self)
		Healcsi:BroadcastEvent("HealcsiGui-DebuffItem-ClearFocus")
		self.state = "selected"
		self.texture:Show()
		self.texture:SetVertexColor(0, 0, 1, 0.3)
		self.fireObj:Fire("OnSelected")
	end)

	for method, func in pairs(methods) do
		widget[method] = func
	end

	return AceGUI:RegisterAsWidget(widget)
end

AceGUI:RegisterWidgetType(Type, Constructor, Version)
