--[[-----------------------------------------------------------------------------
EditBox Widget
-------------------------------------------------------------------------------]]
local Healcsi      = Healcsi
local Gui          = Healcsi.Gui
local TopParent    = Healcsi.UIParent
local HiddenParent = Healcsi.HiddenParent

local Type, Version = "Healcsi-EditBox", 1
local AceGUI = LibStub and LibStub("AceGUI-3.0", true)
if not AceGUI or (AceGUI:GetWidgetVersion(Type) or 0) >= Version then return end

-- Lua APIs
local tostring, pairs = tostring, pairs

-- WoW APIs
local GetCursorInfo, ClearCursor, GetSpellInfo = GetCursorInfo, ClearCursor, GetSpellInfo
local CreateFrame = CreateFrame
local _G = _G

-- Global vars/functions that we don't upvalue since they might get hooked, or upgraded
-- List them here for Mikk's FindGlobals script
-- GLOBALS: AceGUIEditBoxInsertLink, ChatFontNormal, OKAY

--[[-----------------------------------------------------------------------------
Support functions
-------------------------------------------------------------------------------]]
if not AceGUIEditBoxInsertLink then
	-- upgradeable hook
	hooksecurefunc("ChatEdit_InsertLink", function(...) return _G.AceGUIEditBoxInsertLink(...) end)
end

function _G.AceGUIEditBoxInsertLink(text)
	for i = 1, AceGUI:GetWidgetCount(Type) do
		local editbox = _G["AceGUI-3.0EditBox"..i]
		if editbox and editbox:IsVisible() and editbox:HasFocus() then
			editbox:Insert(text)
			return true
		end
	end
end

local function ShowButton(self)
	if not self.disablebutton then
		self.button:Show()
		self.editbox:SetTextInsets(0, 20, 3, 3)
	end
end

local function HideButton(self)
	self.button:Hide()
	self.editbox:SetTextInsets(0, 0, 3, 3)
end

--[[-----------------------------------------------------------------------------
Scripts
-------------------------------------------------------------------------------]]
local function Control_OnEnter(frame)
	frame.obj:Fire("OnEnter")
end

local function Control_OnLeave(frame)
	frame.obj:Fire("OnLeave")
end

local function Frame_OnShowFocus(frame)
	frame.obj.editbox:SetFocus()
	frame:SetScript("OnShow", nil)
end

local function EditBox_OnEscapePressed(frame)
	AceGUI:ClearFocus()
	local self = frame.obj
end

local function EditBox_OnEnterPressed(frame)
	local self = frame.obj
	local value = frame:GetText()
	local cancel = self:Fire("OnEnterPressed", value)
	if not cancel then
		HideButton(self)
	end
end

local function EditBox_OnReceiveDrag(frame)
	local self = frame.obj
	local type, id, info = GetCursorInfo()
	if type == "item" then
		self:SetText(info)
		self:Fire("OnEnterPressed", info)
		ClearCursor()
	elseif type == "spell" then
		local name = GetSpellInfo(id, info)
		self:SetText(name)
		self:Fire("OnEnterPressed", name)
		ClearCursor()
	elseif type == "macro" then
		local name = GetMacroInfo(id)
		self:SetText(name)
		self:Fire("OnEnterPressed", name)
		ClearCursor()
	end
	HideButton(self)
	AceGUI:ClearFocus()
end

local function EditBox_OnTextChanged(frame)
	local self = frame.obj
	local value = frame:GetText()
	if tostring(value) ~= tostring(self.lasttext) then
		self:Fire("OnTextChanged", value)
		self.lasttext = value
		ShowButton(self)
	end
end

local function EditBox_OnFocusGained(frame)
	AceGUI:SetFocus(frame.obj)
end

local function Button_OnClick(frame)
	local editbox = frame.obj.editbox
	editbox:ClearFocus()
	EditBox_OnEnterPressed(editbox)
end

local function Button_OnMouseDown(button)
	button.hiddenState = button.state
	button.state = "pressed"
	button:GetParent():GetParent().obj:ApplyButtonTexture()
end
local function Button_OnMouseUp(button)
	if (button.hiddenState) then
		button.state = button.hiddenState
		button.hiddenState = nil
	else
		button.state = "normal"
	end
	button:GetParent():GetParent().obj:ApplyButtonTexture()
end
local function Button_OnEnter(button)
	if (button.state ~= "pressed") then
		button.state = "hover"
	else
		button.hiddenState = "hover"
	end
	button:GetParent():GetParent().obj:ApplyButtonTexture()
end
local function Button_OnLeave(button)
	if (button.state ~= "pressed") then
		button.state = "normal"
	else
		button.hiddenState = "normal"
	end
	button:GetParent():GetParent().obj:ApplyButtonTexture()
end

--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local methods = {
	["OnAcquire"] = function(self)
		-- height is controlled by SetLabel
		self:SetWidth(200)

		self:SetFont(Healcsi:GetFont("Friz Quadrata TT"))
		self:SetFontColor(Healcsi:GetColor("White", 1))

		self:SetLabelFont(Healcsi:GetFont("MoonBold"))
		self:SetLabelFontColor(Healcsi:GetColor("Black", 1))

		self:SetLabelShadowColor(Healcsi:GetColor("DarkBlue", 0.5))
		self:SetLabelShadowOffset(1, -1)

		self:SetBackgroundTexture("MitsuSaint-Dsm-V7")
		self:SetBackgroundTextureColor(Healcsi:GetColor("DarkBlue", 0.6))

		self:SetButtonFont(Healcsi:GetFont("Friz Quadrata TT"), 12)
		self:SetButtonFontColor(Healcsi:GetColor("White", 1))

		self:SetButtonNormalTexture("LiteStep")
		self:SetButtonHoverTexture("LiteStep")
		self:SetButtonPressedTexture("LiteStep")

		self:SetButtonNormalTextureColor(Healcsi:GetColor("DarkBlue", 0.8))
		self:SetButtonHoverTextureColor(Healcsi:GetColor("BluePressed", 0.8))
		self:SetButtonPressedTextureColor(Healcsi:GetColor("BlueSelected", 0.8))

		self:SetDisabled(false)
		self:SetLabel()
		self:SetText()
		self:DisableButton(false)
		self:SetMaxLetters(0)
	end,

	["OnRelease"] = function(self)
		self:ClearFocus()
	end,

	["SetLabelFont"] = function(self, font, height, flags)
		self.label:SetFont(Healcsi:GetMedia("font", font) or font, height or 13, flags)
	end,

	["SetLabelFontColor"] = function(self, r, g, b, a)
		if not (r and g and b) then
			r, g, b = 0, 0, 0
		end
		self.label.boxTextColor = self.label.boxTextColor or {}
		self.label.boxTextColor.r = r
		self.label.boxTextColor.g = g
		self.label.boxTextColor.b = b
		self.label.boxTextColor.a = a
		self.label:SetVertexColor(r, g, b, a or 1)
	end,

	["SetLabelShadowColor"] = function(self, r, g, b, a)
		self.label:SetShadowColor(r or 0, g or 0, b or 0, a or 1)
		
	end,

	["SetLabelShadowOffset"] = function(self, x, y)
		self.label:SetShadowOffset(x or 0, y or 0)
	end,

	["SetFont"] = function(self, font, height, flags)
		self.editbox:SetFont(Healcsi:GetMedia("font", font) or font, height or 13, flags)
	end,

	["SetFontColor"] = function(self, r, g, b, a)
		if not (r and g and b) then
			r, g, b = 0, 0, 0
		end
		self.editbox.boxTextColor = self.editbox.boxTextColor or {}
		self.editbox.boxTextColor.r = r
		self.editbox.boxTextColor.g = g
		self.editbox.boxTextColor.b = b
		self.editbox.boxTextColor.a = a
		self.editbox:SetTextColor(r, g, b, a or 1)
	end,

	["SetBackgroundTexture"] = function(self, texture)
		self.frame.background:SetTexture(Healcsi:GetMedia("statusbar", texture) or texture)
	end,

	["SetBackgroundTextureColor"] = function(self, r, g, b, a)
		self.frame.background:SetVertexColor(r, g, b, a or 1)
	end,

	["SetButtonNormalTexture"] = function(self, texture)
		self.button.normalTexture = Healcsi:GetMedia("statusbar", texture)
		self:ApplyButtonTexture()
	end,

	["SetButtonNormalTextureColor"] = function(self, r, g, b, a)
		self.button.normalTextureColor = self.button.normalTextureColor or {}
		self.button.normalTextureColor.r = r
		self.button.normalTextureColor.g = g
		self.button.normalTextureColor.b = b
		self.button.normalTextureColor.a = a
		self:ApplyButtonTexture()
	end,

	["SetButtonHoverTexture"] = function(self, texture)
		self.button.hoverTexture = Healcsi:GetMedia("statusbar", texture)
		self:ApplyButtonTexture()
	end,

	["SetButtonHoverTextureColor"] = function(self, r, g, b, a)
		self.button.hoverTextureColor = self.button.hoverTextureColor or {}
		self.button.hoverTextureColor.r = r
		self.button.hoverTextureColor.g = g
		self.button.hoverTextureColor.b = b
		self.button.hoverTextureColor.a = a
		self:ApplyButtonTexture()
	end,

	["SetButtonPressedTexture"] = function(self, texture)
		self.button.pressedTexture = Healcsi:GetMedia("statusbar", texture)
		self:ApplyButtonTexture()
	end,

	["SetButtonPressedTextureColor"] = function(self, r, g, b, a)
		self.button.pressedTextureColor = self.button.pressedTextureColor or {}
		self.button.pressedTextureColor.r = r
		self.button.pressedTextureColor.g = g
		self.button.pressedTextureColor.b = b
		self.button.pressedTextureColor.a = a
		self:ApplyButtonTexture()
	end,

	["ApplyButtonTexture"] = function(self)
		if (self.button.state == "normal") then
			self.button.background:SetTexture(self.button.normalTexture)
			local color = self.button.normalTextureColor or {0, 0, 0}
			self.button.background:SetVertexColor(color.r, color.g, color.b, color.a or 0.9)
		elseif (self.button.state == "hover") then
			self.button.background:SetTexture(self.button.hoverTexture)
			local color = self.button.hoverTextureColor or {0, 0, 0}
			self.button.background:SetVertexColor(color.r, color.g, color.b, color.a or 0.9)
		elseif (self.button.state == "pressed") then
			self.button.background:SetTexture(self.button.pressedTexture)
			local color = self.button.pressedTextureColor or {0, 0, 0}
			self.button.background:SetVertexColor(color.r, color.g, color.b, color.a or 0.9)
		end
	end,

	["SetButtonFont"] = function(self, font, height, flags)
		self.button.text:SetFont(Healcsi:GetMedia("font", font) or font, height or 13, flags)
	end,

	["SetButtonFontColor"] = function(self, r, g, b, a)
		if not (r and g and b) then
			r, g, b = 0.1, 0.1, 0.1
		end
		self.button.text:SetVertexColor(r, g, b, a or 1)

		self.button.text:SetShadowOffset(0,1)
		self.button.text:SetShadowColor(0.8, 0.8, 0.8, 0.2)
	end,

	["SetDisabled"] = function(self, disabled)
		self.disabled = disabled
		if disabled then
			self.editbox:EnableMouse(false)
			self.editbox:ClearFocus()
			self.editbox:SetTextColor(0.5,0.5,0.5)
			self.label:SetTextColor(0.5,0.5,0.5)
		else
			self.editbox:EnableMouse(true)
			local boxColor = self.editbox.boxTextColor or {r=0, g=0, b=0}
			self.editbox:SetTextColor(boxColor.r, boxColor.g, boxColor.b, boxColor.a or 1)
			local labelColor = self.label.boxTextColor or {r=0, g=0, b=0}
			self.label:SetTextColor(labelColor.r, labelColor.g, labelColor.b, labelColor.a or 1)
		end
	end,

	["SetText"] = function(self, text)
		self.lasttext = text or ""
		self.editbox:SetText(text or "")
		self.editbox:SetCursorPosition(0)
		HideButton(self)
	end,

	["GetText"] = function(self, text)
		return self.editbox:GetText()
	end,

	["SetLabel"] = function(self, text)
		if text and text ~= "" then
			self.label:SetText(text)
			self.label:Show()
			self.editbox:SetPoint("TOPLEFT",self.frame,"TOPLEFT",7,-24) -- 7, -20
			self:SetHeight(50)
			self.alignoffset = 30
		else
			self.label:SetText("")
			self.label:Hide()
			self.editbox:SetPoint("TOPLEFT",self.frame,"TOPLEFT",7,0)
			self:SetHeight(26)
			self.alignoffset = 12
		end
	end,

	["DisableButton"] = function(self, disabled)
		self.disablebutton = disabled
		if disabled then
			HideButton(self)
		end
	end,

	["SetMaxLetters"] = function (self, num)
		self.editbox:SetMaxLetters(num or 0)
	end,

	["ClearFocus"] = function(self)
		self.editbox:ClearFocus()
		self.frame:SetScript("OnShow", nil)
	end,

	["SetFocus"] = function(self)
		self.editbox:SetFocus()
		if not self.frame:IsShown() then
			self.frame:SetScript("OnShow", Frame_OnShowFocus)
		end
	end,

	["HighlightText"] = function(self, from, to)
		self.editbox:HighlightText(from, to)
	end
}

--[[-----------------------------------------------------------------------------
Constructor
-------------------------------------------------------------------------------]]
local function Constructor()
	local num  = AceGUI:GetNextWidgetNum(Type)
	local frame = CreateFrame("Frame", nil, TopParent)
	frame:Hide()

	local editbox = CreateFrame("EditBox", "AceGUI-3.0EditBox"..num, frame)
	editbox:SetAutoFocus(false)
	editbox:SetScript("OnEnter", Control_OnEnter)
	editbox:SetScript("OnLeave", Control_OnLeave)
	editbox:SetScript("OnEscapePressed", EditBox_OnEscapePressed)
	editbox:SetScript("OnEnterPressed", EditBox_OnEnterPressed)
	editbox:SetScript("OnTextChanged", EditBox_OnTextChanged)
	editbox:SetScript("OnReceiveDrag", EditBox_OnReceiveDrag)
	editbox:SetScript("OnMouseDown", EditBox_OnReceiveDrag)
	editbox:SetScript("OnEditFocusGained", EditBox_OnFocusGained)
	editbox:SetTextInsets(0, 0, 3, 3)
	editbox:SetMaxLetters(256)
	editbox:SetPoint("BOTTOMLEFT", 6, 0)
	editbox:SetPoint("BOTTOMRIGHT")
	editbox:SetHeight(19)

	local background = editbox:CreateTexture(nil, "BACKGROUND")
	frame.background = background
	background:SetAllPoints(editbox)
	background:SetTexture(Healcsi:GetMedia("statusbar", "Solid"))
	background:SetVertexColor(0, 0, 0, 1)

	local label = frame:CreateFontString(nil, "OVERLAY")
	label:SetPoint("TOPLEFT", 0, -2)
	label:SetPoint("TOPRIGHT", 0, -2)
	label:SetJustifyH("LEFT")
	label:SetHeight(18)

	local button = CreateFrame("Button", nil, editbox)
	button:SetWidth(40)
	button:SetHeight(20)
	button:SetPoint("RIGHT", -2, 0)
	--button:SetText(OKAY)
	button:Hide()

	button:SetScript("OnClick", Button_OnClick)
	button:SetScript("OnMouseDown", Button_OnMouseDown)
	button:SetScript("OnMouseUp", Button_OnMouseUp)
	button:SetScript("OnEnter", Button_OnEnter)
	button:SetScript("OnLeave", Button_OnLeave)
	button.state = "normal"

	local background = button:CreateTexture(nil, "BACKGROUND")
	button.background = background
	background:SetAllPoints(button)
	background:SetTexture(Healcsi:GetMedia("statusbar", "Solid"))
	background:SetVertexColor(0, 0, 0, 1)

	local text = button:CreateFontString(nil, "ARTWORK")
	button.text = text
	text:SetFont(Healcsi:GetMedia("font", "Friz Quadrata TT") ,  13)
	text:SetText(OKAY)
	text:ClearAllPoints()
	text:SetPoint("TOPLEFT", 1, -1)
	text:SetPoint("BOTTOMRIGHT", -1, 1)
	text:SetJustifyV("MIDDLE")

	local widget = {
		alignoffset = 30,
		editbox     = editbox,
		label       = label,
		button      = button,
		frame       = frame,
		type        = Type
	}
	for method, func in pairs(methods) do
		widget[method] = func
	end
	editbox.obj, button.obj = widget, widget

	return AceGUI:RegisterAsWidget(widget)
end

AceGUI:RegisterWidgetType(Type, Constructor, Version)
