
local Healcsi = Healcsi
local Media   = Healcsi.Media

-- Healcsi:GetMedia(mediaType, mediaName)
-- Healcsi:SetTexture(texture, mediaType, mediaName)
-- Healcsi:SetVertexColor(texture, colorName, colorAlpha)
-- Healcsi:DecorateTexture(texture, mediaType, mediaName, colorName, colorAlpha)
-- Healcsi:SetPadding(box, parentBox, padding)
-- Healcsi:CreateTextureFull(frame, layer, padding)
-- Healcsi:CreateTextureHoriz(frame, layer, sizePadding, top, height)

function Healcsi:GetMedia(mediaType, mediaName)
	return self.Media:Fetch(mediaType, mediaName) or mediaName
end

function Healcsi:GetFont(fontName)
	return self.Media:Fetch("font", fontName) or fontName
end

function Healcsi:SetTexture(texture, mediaType, mediaName)
	texture:SetTexture(self:GetMedia(mediaType, mediaName))
end

function Healcsi:SetVertexColor(texture, colorName, colorAlpha)
	texture:SetVertexColor(self:GetColor(colorName, colorAlpha or 1))
end

function Healcsi:DecorateTexture(texture, mediaType, mediaName, colorName, colorAlpha)
	self:SetTexture(texture, mediaType, mediaName)
	self:SetVertexColor(texture, colorName, colorAlpha)
end

function Healcsi:IsLayerValid(layer)
	if (layer == "BACKGROUND") or (layer == "BORDER") or (layer == "ARTWORK") or (layer == "OVERLAY") or (layer == "HIGHLIGHT") then
		return true
	end
	return false
end

function Healcsi:SetPadding(box, parentBox, padding)
	if (box == nil) or (parentBox == nil) then return end

	padding = padding or {0, 0, 0, 0}

	local top     = padding[1] or 0
	local right   = padding[2] or 0
	local bottom  = padding[3] or 0
	local left    = padding[4] or 0

	box:ClearAllPoints()
	box:SetPoint("TOPLEFT", parentBox, "TOPLEFT", left, -top)
	box:SetPoint("BOTTOMRIGHT", parentBox, "BOTTOMRIGHT", -right, bottom)
end

function Healcsi:CreateTextureFull(frame, layer, padding)
	if (not self:IsLayerValid(layer)) or (frame == nil) then
		return nil
	end

	local texture = frame:CreateTexture(nil, layer)
	self:SetPadding(texture, frame, padding)
	return texture
end

function Healcsi:SetLeftLine(tex, frame, width)
	tex:ClearAllPoints()
	tex:SetPoint("TOPLEFT", frame, "TOPLEFT", 3, 0)
	tex:SetPoint("BOTTOMRIGHT", frame, "BOTTOMLEFT", width+3, 0)
end

function Healcsi:SetRightLine(tex, frame, width)
	tex:ClearAllPoints()
	tex:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -3, 0)
	tex:SetPoint("BOTTOMLEFT", frame, "BOTTOMRIGHT", -width-3, 0)
end

function Healcsi:CreateLeftLineTexture(frame, layer, width)
	if (not self:IsLayerValid(layer)) or (frame == nil) then
		return nil
	end

	local texture = frame:CreateTexture(nil, layer)
	self:SetLeftLine(texture, frame, width)
	return texture
end

function Healcsi:CreateRightLineTexture(frame, layer, width)
	if (not self:IsLayerValid(layer)) or (frame == nil) then
		return nil
	end

	local texture = frame:CreateTexture(nil, layer)
	self:SetRightLine(texture, frame, width)
	return texture
end

function Healcsi:CreateTextureHoriz(frame, layer, sizePadding, top, height)
	if (not self:IsLayerValid(layer)) or (frame == nil) then
		return nil
	end

	local texture = frame:CreateTexture(nil, layer)

	sizePadding = sizePadding or {0, 0}
	local left  = sizePadding[1] or 0
	local right = sizePadding[2] or 0

	texture:SetPoint("TOPLEFT", frame, "TOPLEFT", left, -top)
	texture:SetPoint("BOTTOMRIGHT", frame, "TOPRIGHT", -right, -top - height)
	return texture
end

function Healcsi:CreateTwoTexturesHoriz(frame, layer, sizePadding, top, height)
	if (not self:IsLayerValid(layer)) or (frame == nil) then
		return nil
	end

	local horizFrame = CreateFrame("Frame", nil, frame)

	sizePadding = sizePadding or {0, 0}
	local left  = sizePadding[1] or 0
	local right = sizePadding[2] or 0

	horizFrame:SetPoint("TOPLEFT", frame, "TOPLEFT", left, -top)
	horizFrame:SetPoint("BOTTOMRIGHT", frame, "TOPRIGHT", -right, -top - height)

	local leftTexture  = frame:CreateTexture(nil, layer)
	local rightTexture = frame:CreateTexture(nil, layer)

	leftTexture:SetPoint("TOPLEFT", horizFrame)
	leftTexture:SetPoint("BOTTOMRIGHT", horizFrame, "BOTTOMLEFT", horizFrame:GetWidth()/2, 0)

	rightTexture:SetPoint("TOPRIGHT", horizFrame)
	rightTexture:SetPoint("BOTTOMLEFT", horizFrame, "BOTTOMRIGHT",-horizFrame:GetWidth()/2, 0)

	horizFrame.textures = {}
	horizFrame.textures.left  = leftTexture
	horizFrame.textures.right = rightTexture

	return horizFrame
end

function Healcsi:UpdateMiddleBackground(frame, texture, mediaWidth, mediaHeight, padding)
	local width, height = frame:GetSize()
	local widthRatio    = mediaWidth  / width
	local heightRatio   = mediaHeight / height
	local ratio = mediaWidth / mediaHeight

	padding = padding or 0

	texture:ClearAllPoints()

	if (heightRatio > widthRatio) then
		-- scale image to match the height, and let the width have the gaps
		local newWidth   = ratio * (height - (2*padding))
		local pointValue = (width - newWidth) / 2

		texture:SetPoint("TOPLEFT", frame, "TOPLEFT", pointValue, -padding)
		texture:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -pointValue, padding)

	else
		-- scale image to match the width, and let the height have the gaps
		local newHeight  = (width - (2*padding)) / ratio
		local pointValue = (height - newHeight) / 2

		texture:SetPoint("TOPLEFT", frame, "TOPLEFT", padding, -pointValue)
		texture:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -padding, pointValue)
	end

end

function Healcsi:SetMiddleBackground(frame, layer, mediaName, mediaWidth, mediaHeight, padding, image2Power)
	if (not self:IsLayerValid(layer)) or (frame == nil) or (mediaWidth == nil) or (mediaHeight == nil) then
		return nil
	end

	local texture = frame:CreateTexture(nil, layer)
	self:SetTexture(texture, "background", mediaName)

	local xRatio = (mediaWidth  / image2Power)
	local yRatio = (mediaHeight / image2Power)

	local left  = (1-xRatio)/2
	local right = 1- left

	local top = (1-yRatio)/2
	local bottom = 1-top

	texture:SetTexCoord(left, right, top, bottom)

	self:UpdateMiddleBackground(frame, texture, mediaWidth, mediaHeight, padding or 0)
	return texture
end

function Healcsi:SetFrameShadow(frame, texture)
	if (frame.shadow == nil) then self:CreateFrameShadow(frame, texture) end

	local shadowTexture = Healcsi:GetMedia("shadow", texture)

	frame.shadow.left:SetTexture(shadowTexture)
	frame.shadow.left:SetTexCoord(0, 0.25, 0.25, 0.75)
	frame.shadow.left:SetWidth(128)

	frame.shadow.right:SetTexture(shadowTexture)
	frame.shadow.right:SetTexCoord(0.75, 1, 0.25, 0.75)
	frame.shadow.right:SetWidth(128)

	frame.shadow.top:SetTexture(shadowTexture)
	frame.shadow.top:SetTexCoord(0.25, 0.75, 0, 0.25)
	frame.shadow.top:SetHeight(128)

	frame.shadow.bottom:SetTexture(shadowTexture)
	frame.shadow.bottom:SetTexCoord(0.25, 0.75, 0.75, 1)
	frame.shadow.bottom:SetHeight(128)

	frame.shadow.topLeft:SetTexture(shadowTexture)
	frame.shadow.topLeft:SetTexCoord(0, 0.25, 0, 0.25)
	frame.shadow.topLeft:SetSize(128, 128)

	frame.shadow.topRight:SetTexture(shadowTexture)
	frame.shadow.topRight:SetTexCoord(0.75, 1, 0, 0.25)
	frame.shadow.topRight:SetSize(128, 128)

	frame.shadow.bottomLeft:SetTexture(shadowTexture)
	frame.shadow.bottomLeft:SetTexCoord(0, 0.24, 0.75, 1)
	frame.shadow.bottomLeft:SetSize(128, 128)

	frame.shadow.bottomRight:SetTexture(shadowTexture)
	frame.shadow.bottomRight:SetTexCoord(0.75, 1, 0.75, 1)
	frame.shadow.bottomRight:SetSize(128, 128)
end

function Healcsi:SetFrameShadowColor(frame, r, g, b, a)
	if (frame.shadow == nil) then self:CreateFrameShadow(frame, texture) end

	frame.shadow.left:SetVertexColor(r, g, b, a or 1)
	frame.shadow.right:SetVertexColor(r, g, b, a or 1)
	frame.shadow.top:SetVertexColor(r, g, b, a or 1)
	frame.shadow.bottom:SetVertexColor(r, g, b, a or 1)
	frame.shadow.topLeft:SetVertexColor(r, g, b, a or 1)
	frame.shadow.topRight:SetVertexColor(r, g, b, a or 1)
	frame.shadow.bottomLeft:SetVertexColor(r, g, b, a or 1)
	frame.shadow.bottomRight:SetVertexColor(r, g, b, a or 1)
end

function Healcsi:CreateFrameShadow(frame, texture)
	
	local leftShadow = frame:CreateTexture(nil, "BORDER")
	leftShadow:SetPoint("TOPRIGHT", frame, "TOPLEFT")
	leftShadow:SetPoint("BOTTOMRIGHT", frame, "BOTTOMLEFT")

	local rightShadow = frame:CreateTexture(nil, "BORDER")
	rightShadow:SetPoint("TOPLEFT", frame, "TOPRIGHT")
	rightShadow:SetPoint("BOTTOMLEFT", frame, "BOTTOMRIGHT")

	local topShadow = frame:CreateTexture(nil, "BORDER")
	topShadow:SetPoint("BOTTOMLEFT", frame, "TOPLEFT")
	topShadow:SetPoint("BOTTOMRIGHT", frame, "TOPRIGHT")

	local bottomShadow = frame:CreateTexture(nil, "BORDER")
	bottomShadow:SetPoint("TOPLEFT", frame, "BOTTOMLEFT")
	bottomShadow:SetPoint("TOPRIGHT", frame, "BOTTOMRIGHT")

	frame.shadow = frame.shadow or {}

	frame.shadow.left = leftShadow
	frame.shadow.right = rightShadow
	frame.shadow.top = topShadow
	frame.shadow.bottom = bottomShadow

	local topLeftShadow = frame:CreateTexture(nil, "BORDER")
	topLeftShadow:SetPoint("BOTTOMRIGHT", frame, "TOPLEFT")

	local topRightShadow = frame:CreateTexture(nil, "BORDER")
	topRightShadow:SetPoint("BOTTOMLEFT", frame, "TOPRIGHT")

	local bottomLeftShadow = frame:CreateTexture(nil, "BORDER")
	bottomLeftShadow:SetPoint("TOPRIGHT", frame, "BOTTOMLEFT")

	local bottomRightShadow = frame:CreateTexture(nil, "BORDER")
	bottomRightShadow:SetPoint("TOPLEFT", frame, "BOTTOMRIGHT")

	frame.shadow.topLeft = topLeftShadow
	frame.shadow.topRight = topRightShadow
	frame.shadow.bottomLeft = bottomLeftShadow
	frame.shadow.bottomRight = bottomRightShadow
end

Healcsi.Media.borderTextures = {
	["left-1"] = [[Interface\Addons\Healcsi\Media\UI\dividers\divider-1-left]],
	["left-2"] = [[Interface\Addons\Healcsi\Media\UI\dividers\divider-2-left]],
	["left-3"] = [[Interface\Addons\Healcsi\Media\UI\dividers\divider-3-left]],
	["left-4"] = [[Interface\Addons\Healcsi\Media\UI\dividers\divider-4-left]],
	["left-5"] = [[Interface\Addons\Healcsi\Media\UI\dividers\divider-5-left]],

	["right-1"] = [[Interface\Addons\Healcsi\Media\UI\dividers\divider-1-right]],
	["right-2"] = [[Interface\Addons\Healcsi\Media\UI\dividers\divider-2-right]],
	["right-3"] = [[Interface\Addons\Healcsi\Media\UI\dividers\divider-3-right]],
	["right-4"] = [[Interface\Addons\Healcsi\Media\UI\dividers\divider-4-right]],
	["right-5"] = [[Interface\Addons\Healcsi\Media\UI\dividers\divider-5-right]],

	["top-1"] = [[Interface\Addons\Healcsi\Media\UI\dividers\divider-1-top]],
	["top-2"] = [[Interface\Addons\Healcsi\Media\UI\dividers\divider-2-top]],
	["top-3"] = [[Interface\Addons\Healcsi\Media\UI\dividers\divider-3-top]],
	["top-4"] = [[Interface\Addons\Healcsi\Media\UI\dividers\divider-4-top]],
	["top-5"] = [[Interface\Addons\Healcsi\Media\UI\dividers\divider-5-top]],

	["bottom-1"] = [[Interface\Addons\Healcsi\Media\UI\dividers\divider-1-bottom]],
	["bottom-2"] = [[Interface\Addons\Healcsi\Media\UI\dividers\divider-2-bottom]],
	["bottom-3"] = [[Interface\Addons\Healcsi\Media\UI\dividers\divider-3-bottom]],
	["bottom-4"] = [[Interface\Addons\Healcsi\Media\UI\dividers\divider-4-bottom]],
	["bottom-5"] = [[Interface\Addons\Healcsi\Media\UI\dividers\divider-5-bottom]],

	["None"] = nil
}

Healcsi.Media.aggroTextures = {
	["threat-1"] = [[Interface\Addons\Healcsi\Media\UI\threat\threat-1]],
	["threat-2"] = [[Interface\Addons\Healcsi\Media\UI\threat\threat-2]],
	["threat-3"] = [[Interface\Addons\Healcsi\Media\UI\threat\threat-3]],
	["threat-4"] = [[Interface\Addons\Healcsi\Media\UI\threat\threat-4]],
	["threat-5"] = [[Interface\Addons\Healcsi\Media\UI\threat\threat-5]],
	["threat-6"] = [[Interface\Addons\Healcsi\Media\UI\threat\threat-6]],
	["threat-7"] = [[Interface\Addons\Healcsi\Media\UI\threat\threat-7]]
}

Healcsi.Media.selectionTextures = {
	["select-1"] = [[Interface\Addons\Healcsi\Media\UI\select\select-1]],
	["select-2"] = [[Interface\Addons\Healcsi\Media\UI\select\select-2]],
	["select-3"] = [[Interface\Addons\Healcsi\Media\UI\select\select-3]],
	["select-4"] = [[Interface\Addons\Healcsi\Media\UI\select\select-4]],
	["select-5"] = [[Interface\Addons\Healcsi\Media\UI\select\select-5]],
	["select-6"] = [[Interface\Addons\Healcsi\Media\UI\select\select-6]],
	["select-7"] = [[Interface\Addons\Healcsi\Media\UI\select\select-7]]
}

Healcsi.Media.absorbTextures = {
	["absorb-1"] = [[Interface\Addons\Healcsi\Media\UI\absorb\absorb-1]],
	["absorb-2"] = [[Interface\Addons\Healcsi\Media\UI\absorb\absorb-2]],
	["absorb-3"] = [[Interface\Addons\Healcsi\Media\UI\absorb\absorb-3]],
	["absorb-4"] = [[Interface\Addons\Healcsi\Media\UI\absorb\absorb-4]],
	["absorb-5"] = [[Interface\Addons\Healcsi\Media\UI\absorb\absorb-5]],
	["absorb-6"] = [[Interface\Addons\Healcsi\Media\UI\absorb\absorb-6]],
	["absorb-7"] = [[Interface\Addons\Healcsi\Media\UI\absorb\absorb-7]],
	["absorb-8"] = [[Interface\Addons\Healcsi\Media\UI\absorb\absorb-8]],
	["absorb-9"] = [[Interface\Addons\Healcsi\Media\UI\absorb\absorb-9]],
	["absorb-10"] = [[Interface\Addons\Healcsi\Media\UI\absorb\absorb-10]],
	["absorb-11"] = [[Interface\Addons\Healcsi\Media\UI\absorb\absorb-11]],
	["absorb-12"] = [[Interface\Addons\Healcsi\Media\UI\absorb\absorb-12]]
}

Healcsi.Media.bgTextures = {
	["shaman-1"] = [[Interface\Addons\Healcsi\Media\UI\background]],
	["priest-1"] = [[Interface\Addons\Healcsi\Media\UI\background2]],
	["paladin-1"] = [[Interface\Addons\Healcsi\Media\UI\background3]],
}

Healcsi.Media.bgWidth = {
	["shaman-1"] = 1024,
	["priest-1"] = 1024,
	["paladin-1"] = 1024,
}

Healcsi.Media.bgHeight = {
	["shaman-1"] = 793,
	["priest-1"] = 713,
	["paladin-1"] = 700
}

-- Register some SharedMedia goodies.

Media:Register("statusbar", "Solid", [[Interface\Buttons\WHITE8X8]])
Media:Register("border",    "Solid", [[Interface\Buttons\WHITE8X8]])

Media:Register("shadow", "Shadow-1", [[Interface\Addons\Healcsi\Media\UI\Shadow1]])
Media:Register("shadow", "Shadow-2", [[Interface\Addons\Healcsi\Media\UI\Shadow2]])
Media:Register("shadow", "Shadow-3", [[Interface\Addons\Healcsi\Media\UI\Shadow3]])

Media:Register("font", "MoonBold",  [[Interface\Addons\Healcsi\Media\UI\MoonBold.ttf]])
Media:Register("font", "MoonLight", [[Interface\Addons\Healcsi\Media\UI\MoonLight.ttf]])
Media:Register("font", "Collegiate",  [[Interface\Addons\Healcsi\Media\UI\COLLEGIA.ttf]])
Media:Register("font", "Impact",    [[Interface\Addons\Healcsi\Media\UI\impact.ttf]])

-- Media:Register("background", "ShamanBackground", [[Interface\Addons\Healcsi\Media\UI\background]]) -- 1024 x 793
-- Media:Register("background", "HeadLine", [[Interface\Addons\Healcsi\Media\UI\headLine]]) -- 16 x 1024
-- Media:Register("background", "Close", [[Interface\Addons\Healcsi\Media\UI\close]]) -- 512 x 512
-- Media:Register("background", "Resize", [[Interface\Addons\Healcsi\Media\UI\resize]]) -- 256 x 256
-- Media:Register("background", "MenuSeparator", [[Interface\Addons\Healcsi\Media\UI\menu]]) -- 4 x 32
-- Media:Register("background", "MenuSelector", [[Interface\Addons\Healcsi\Media\UI\selector]]) -- 4 x 32
Media:Register("background", "DownArrow", [[Interface\Addons\Healcsi\Media\UI\down]]) -- 256 x 256
Media:Register("background", "DownArrowSmall", [[Interface\Addons\Healcsi\Media\UI\smalldown]]) -- 256 x 256
-- Media:Register("background", "CheckBox", [[Interface\Addons\Healcsi\Media\UI\checkbox]]) -- 256 x 256
-- Media:Register("background", "Checked", [[Interface\Addons\Healcsi\Media\UI\checked]]) -- 256 x 256
-- Media:Register("background", "CheckBoxHighLight", [[Interface\Addons\Healcsi\Media\UI\checkboxhighlight]]) -- 256 x 256
-- Media:Register("background", "Swatch", [[Interface\Addons\Healcsi\Media\ColorPicker\swatch]]) -- 256 x 256
-- Media:Register("background", "Slider", [[Interface\Addons\Healcsi\Media\UI\slider]]) -- 256 x 32

Media:Register("statusbar", "Henna-Comity",      [[Interface\Addons\Healcsi\Media\Henna\statusbar\Comity]])
Media:Register("statusbar", "Henna-Courtesy",    [[Interface\Addons\Healcsi\Media\Henna\statusbar\Courtesy]])
Media:Register("statusbar", "Henna-Parity",      [[Interface\Addons\Healcsi\Media\Henna\statusbar\Parity]])
Media:Register("statusbar", "Henna-Reliance",    [[Interface\Addons\Healcsi\Media\Henna\statusbar\Reliance]])
Media:Register("statusbar", "Henna-Solicitude",  [[Interface\Addons\Healcsi\Media\Henna\statusbar\Solicitude]])
Media:Register("statusbar", "Henna-Tolerance",   [[Interface\Addons\Healcsi\Media\Henna\statusbar\Tolerance]])
Media:Register("statusbar", "Henna-Veracity",    [[Interface\Addons\Healcsi\Media\Henna\statusbar\Veracity]])

Media:Register("statusbar", "Aluminium",         [[Interface\Addons\Healcsi\Media\Skada\statusbar\Aluminium]])
Media:Register("statusbar", "Armory",            [[Interface\Addons\Healcsi\Media\Skada\statusbar\Armory]])
Media:Register("statusbar", "BantoBar",          [[Interface\Addons\Healcsi\Media\Skada\statusbar\BantoBar]])
Media:Register("statusbar", "Glaze2",            [[Interface\Addons\Healcsi\Media\Skada\statusbar\Glaze2]])
Media:Register("statusbar", "Gloss",             [[Interface\Addons\Healcsi\Media\Skada\statusbar\Gloss]])
Media:Register("statusbar", "Graphite",          [[Interface\Addons\Healcsi\Media\Skada\statusbar\Graphite]])
Media:Register("statusbar", "Grid",              [[Interface\Addons\Healcsi\Media\Skada\statusbar\Grid]])
Media:Register("statusbar", "Healbot",           [[Interface\Addons\Healcsi\Media\Skada\statusbar\Healbot]])
Media:Register("statusbar", "LiteStep",          [[Interface\Addons\Healcsi\Media\Skada\statusbar\LiteStep]])
Media:Register("statusbar", "Minimalist",        [[Interface\Addons\Healcsi\Media\Skada\statusbar\Minimalist]])
Media:Register("statusbar", "TukTex",            [[Interface\Addons\Healcsi\Media\Skada\statusbar\normTex]])
Media:Register("statusbar", "Otravi",            [[Interface\Addons\Healcsi\Media\Skada\statusbar\Otravi]])
Media:Register("statusbar", "Outline",           [[Interface\Addons\Healcsi\Media\Skada\statusbar\Outline]])
Media:Register("statusbar", "Perl",              [[Interface\Addons\Healcsi\Media\Skada\statusbar\Perl]])
Media:Register("statusbar", "Round",             [[Interface\Addons\Healcsi\Media\Skada\statusbar\Round]])
Media:Register("statusbar", "Smooth",            [[Interface\Addons\Healcsi\Media\Skada\statusbar\Smooth]])

Media:Register("border", "Glow",         [[Interface\Addons\Healcsi\Media\Skada\border\glowTex]])
Media:Register("border", "Roth",         [[Interface\Addons\Healcsi\Media\Skada\border\roth]])

Media:Register("background", "Copper",   [[Interface\Addons\Healcsi\Media\Skada\background\copper]])

Media:Register("font", "ABF",                     [[Interface\Addons\Healcsi\Media\Skada\fonts\ABF]])
Media:Register("font", "Accidental Presidency",   [[Interface\Addons\Healcsi\Media\Skada\fonts\Accidental Presidency]])
Media:Register("font", "Adventure",               [[Interface\Addons\Healcsi\Media\Skada\fonts\Adventure]])
Media:Register("font", "Diablo",                  [[Interface\Addons\Healcsi\Media\Skada\fonts\Avqest]])
Media:Register("font", "Vera Serif",              [[Interface\Addons\Healcsi\Media\Skada\fonts\VeraSe]])

Media:Register("statusbar", "MitsuSaint-Bars",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Bars]])
Media:Register("statusbar", "MitsuSaint-Bumps",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Bumps]])
Media:Register("statusbar", "MitsuSaint-Button",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Button]])
Media:Register("statusbar", "MitsuSaint-Cabaret-1",        [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\cabaret1]])
Media:Register("statusbar", "MitsuSaint-Cabaret-2",        [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\cabaret2]])
Media:Register("statusbar", "MitsuSaint-Cabaret-3",        [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\cabaret3]])
Media:Register("statusbar", "MitsuSaint-Cabaret-4",        [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\cabaret4]])
Media:Register("statusbar", "MitsuSaint-Cabaret-5",        [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\cabaret5]])
Media:Register("statusbar", "MitsuSaint-Charcoal",         [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Charcoal]])
Media:Register("statusbar", "MitsuSaint-Cilo",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Cilo]])
Media:Register("statusbar", "MitsuSaint-Cloud",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Cloud]])
Media:Register("statusbar", "MitsuSaint-Combo",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\combo]])
Media:Register("statusbar", "MitsuSaint-Comet",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Comet]])
Media:Register("statusbar", "MitsuSaint-Dabs",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Dabs]])
Media:Register("statusbar", "MitsuSaint-DarkBottom",       [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DarkBottom]])
Media:Register("statusbar", "MitsuSaint-Diagonal",         [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Diagonal]])
Media:Register("statusbar", "MitsuSaint-Dsm-B1",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DsmB1]])
Media:Register("statusbar", "MitsuSaint-Dsm-B2",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DsmB2]])
Media:Register("statusbar", "MitsuSaint-Dsm-B3",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DsmB3]])
Media:Register("statusbar", "MitsuSaint-Dsm-B4",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DsmB4]])
Media:Register("statusbar", "MitsuSaint-Dsm-B5",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DsmB5]])
Media:Register("statusbar", "MitsuSaint-Dsm-B6",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DsmB6]])
Media:Register("statusbar", "MitsuSaint-Dsm-B7",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DsmB7]])
Media:Register("statusbar", "MitsuSaint-Dsm-B8",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DsmB8]])
Media:Register("statusbar", "MitsuSaint-Dsm-B9",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DsmB9]])
Media:Register("statusbar", "MitsuSaint-DsmOpaque-V1",     [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DsmOpaqueV1]])
Media:Register("statusbar", "MitsuSaint-DsmOpaque-V2",     [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DsmOpaqueV2]])
Media:Register("statusbar", "MitsuSaint-DsmOpaque-V3",     [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DsmOpaqueV3]])
Media:Register("statusbar", "MitsuSaint-DsmOpaque-V4",     [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DsmOpaqueV4]])
Media:Register("statusbar", "MitsuSaint-DsmOpaque-V5",     [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DsmOpaqueV5]])
Media:Register("statusbar", "MitsuSaint-DsmOpaque-V6",     [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DsmOpaqueV6]])
Media:Register("statusbar", "MitsuSaint-DsmOpaque-V7",     [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DsmOpaqueV7]])
Media:Register("statusbar", "MitsuSaint-DsmOpaque-V8",     [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DsmOpaqueV8]])
Media:Register("statusbar", "MitsuSaint-Dsm-V1",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DsmV1]])
Media:Register("statusbar", "MitsuSaint-Dsm-V2",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DsmV2]])
Media:Register("statusbar", "MitsuSaint-Dsm-V3",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DsmV3]])
Media:Register("statusbar", "MitsuSaint-Dsm-V4",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DsmV4]])
Media:Register("statusbar", "MitsuSaint-Dsm-V5",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DsmV5]])
Media:Register("statusbar", "MitsuSaint-Dsm-V6",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DsmV6]])
Media:Register("statusbar", "MitsuSaint-Dsm-V7",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DsmV7]])
Media:Register("statusbar", "MitsuSaint-Dsm-V8",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DsmV8]])
Media:Register("statusbar", "MitsuSaint-Dsm-V9",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DsmV9]])
Media:Register("statusbar", "MitsuSaint-DukeB",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DukeB]])
Media:Register("statusbar", "MitsuSaint-DukeG",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\DukeG]])
Media:Register("statusbar", "MitsuSaint-Empty",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Empty]])
Media:Register("statusbar", "MitsuSaint-Fade1",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\fade1]])
Media:Register("statusbar", "MitsuSaint-Fade2",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\fade2]])
Media:Register("statusbar", "MitsuSaint-Falumn",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Falumn]])
Media:Register("statusbar", "MitsuSaint-Fifths",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Fifths]])
Media:Register("statusbar", "MitsuSaint-Flat",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Flat]])
Media:Register("statusbar", "MitsuSaint-Fourths",          [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Fourths]])
Media:Register("statusbar", "MitsuSaint-Frost",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Frost]])
Media:Register("statusbar", "MitsuSaint-Glamour-1",        [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Glamour]])
Media:Register("statusbar", "MitsuSaint-Glamour-2",        [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Glamour2]])
Media:Register("statusbar", "MitsuSaint-Glamour-3",        [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Glamour3]])
Media:Register("statusbar", "MitsuSaint-Glamour-4",        [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Glamour4]])
Media:Register("statusbar", "MitsuSaint-Glamour-5",        [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Glamour5]])
Media:Register("statusbar", "MitsuSaint-Glamour-6",        [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Glamour6]])
Media:Register("statusbar", "MitsuSaint-Glamour-7",        [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Glamour7]])
Media:Register("statusbar", "MitsuSaint-Glass",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Glass]])
Media:Register("statusbar", "MitsuSaint-Glaze-1",          [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Glaze]])
Media:Register("statusbar", "MitsuSaint-HalA",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\HalA]])
Media:Register("statusbar", "MitsuSaint-HalB",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\HalB]])
Media:Register("statusbar", "MitsuSaint-HalC",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\HalC]])
Media:Register("statusbar", "MitsuSaint-HalD",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\HalD]])
Media:Register("statusbar", "MitsuSaint-HalE",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\HalE]])
Media:Register("statusbar", "MitsuSaint-HalF",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\HalF]])
Media:Register("statusbar", "MitsuSaint-HalG",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\HalG]])
Media:Register("statusbar", "MitsuSaint-HalH",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\HalH]])
Media:Register("statusbar", "MitsuSaint-HalI",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\HalI]])
Media:Register("statusbar", "MitsuSaint-HalJ",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\HalJ]])
Media:Register("statusbar", "MitsuSaint-HalK",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\HalK]])
Media:Register("statusbar", "MitsuSaint-HalL",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\HalL]])
Media:Register("statusbar", "MitsuSaint-HalM",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\HalM]])
Media:Register("statusbar", "MitsuSaint-HalN",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\HalN]])
Media:Register("statusbar", "MitsuSaint-HalO",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\HalO]])
Media:Register("statusbar", "MitsuSaint-HalP",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\HalP]])
Media:Register("statusbar", "MitsuSaint-HalQ",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\HalQ]])
Media:Register("statusbar", "MitsuSaint-HalR",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\HalR]])
Media:Register("statusbar", "MitsuSaint-HalS",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\HalS]])
Media:Register("statusbar", "MitsuSaint-HalT",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\HalT]])
Media:Register("statusbar", "MitsuSaint-HalU",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\HalU]])
Media:Register("statusbar", "MitsuSaint-HalV",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\HalV]])
Media:Register("statusbar", "MitsuSaint-HalW",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\HalW]])
Media:Register("statusbar", "MitsuSaint-HalX",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\HalX]])
Media:Register("statusbar", "MitsuSaint-Hatched",          [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Hatched]])
Media:Register("statusbar", "MitsuSaint-Ish-0",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Ish00]])
Media:Register("statusbar", "MitsuSaint-Ish-1",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Ish01]])
Media:Register("statusbar", "MitsuSaint-Ish-2",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Ish02]])
Media:Register("statusbar", "MitsuSaint-Ish-3",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Ish03]])
Media:Register("statusbar", "MitsuSaint-Ish-4",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Ish04]])
Media:Register("statusbar", "MitsuSaint-Ish-5",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Ish05]])
Media:Register("statusbar", "MitsuSaint-Ish-6",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Ish06]])
Media:Register("statusbar", "MitsuSaint-Ish-7",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Ish07]])
Media:Register("statusbar", "MitsuSaint-Ish-8",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Ish08]])
Media:Register("statusbar", "MitsuSaint-Ish-9",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Ish09]])
Media:Register("statusbar", "MitsuSaint-Ish-10",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Ish10]])
Media:Register("statusbar", "MitsuSaint-Ish-11",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Ish11]])
Media:Register("statusbar", "MitsuSaint-Ish-12",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Ish12]])
Media:Register("statusbar", "MitsuSaint-Ish-13",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Ish13]])
Media:Register("statusbar", "MitsuSaint-Ish-14",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Ish14]])
Media:Register("statusbar", "MitsuSaint-LiteStepLite",     [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\LiteStepLite]])
Media:Register("statusbar", "MitsuSaint-Lyfe",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Lyfe]])
Media:Register("statusbar", "MitsuSaint-Melli",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Melli]])
Media:Register("statusbar", "MitsuSaint-MelliDark",        [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\MelliDark]])
Media:Register("statusbar", "MitsuSaint-MelliDarkRough",   [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\MelliDarkRough]])
Media:Register("statusbar", "MitsuSaint-Perl-2",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Perl2]])
Media:Register("statusbar", "MitsuSaint-Pill",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Pill]])
Media:Register("statusbar", "MitsuSaint-Rain",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Rain]])
Media:Register("statusbar", "MitsuSaint-Rocks",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Rocks]])
Media:Register("statusbar", "MitsuSaint-Ruben",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Ruben]])
Media:Register("statusbar", "MitsuSaint-Runes",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Runes]])
Media:Register("statusbar", "MitsuSaint-Savant-1",         [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\savant1]])
Media:Register("statusbar", "MitsuSaint-Savant-2",         [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\savant2]])
Media:Register("statusbar", "MitsuSaint-Savant-3",         [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\savant3]])
Media:Register("statusbar", "MitsuSaint-Savant-4",         [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\savant4]])
Media:Register("statusbar", "MitsuSaint-Savant-5",         [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\savant5]])
Media:Register("statusbar", "MitsuSaint-Savant-6",         [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\savant6]])
Media:Register("statusbar", "MitsuSaint-Savant-7",         [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\savant7]])
Media:Register("statusbar", "MitsuSaint-Savant-8",         [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\savant8]])
Media:Register("statusbar", "MitsuSaint-Skewed",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Skewed]])
Media:Register("statusbar", "MitsuSaint-Smooth-2",         [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Smoothv2]])
Media:Register("statusbar", "MitsuSaint-Smudge",           [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Smudge]])
Media:Register("statusbar", "MitsuSaint-Steel",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Steel]])
Media:Register("statusbar", "MitsuSaint-Striped",          [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Striped]])
Media:Register("statusbar", "MitsuSaint-Tap-1",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\tap1]])
Media:Register("statusbar", "MitsuSaint-Tap-2",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\tap2]])
Media:Register("statusbar", "MitsuSaint-Tap-3",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\tap3]])
Media:Register("statusbar", "MitsuSaint-Tap-4",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\tap4]])
Media:Register("statusbar", "MitsuSaint-Tube",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Tube]])
Media:Register("statusbar", "MitsuSaint-Water",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Water]])
Media:Register("statusbar", "MitsuSaint-W-Glass",          [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Wglass]])
Media:Register("statusbar", "MitsuSaint-Wisps",            [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Wisps]])
Media:Register("statusbar", "MitsuSaint-Xeon",             [[Interface\Addons\Healcsi\Media\MitsuSaint\statusbar\Xeon]])
